package rexsee.barcode;

import java.util.Collection;
import java.util.HashSet;

import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Rect;
import android.view.View;

import com.google.zxing.ResultPoint;

public class ViewfinderView extends View {
	/**
	 * This view is overlaid on top of the camera preview. It adds the viewfinder rectangle and partial
	 * transparency outside it, as well as the laser scanner animation and result points.
	 */

	private static final int[] SCANNER_ALPHA = {0, 64, 128, 192, 255, 192, 128, 64};
	private static final long ANIMATION_DELAY = 100L;
	private static final int OPAQUE = 0xFF;

	private final BarcodeCaptureDialog mDialog;
	private final Paint paint = new Paint();

	private int scannerAlpha = 0;
	private Bitmap resultBitmap;
	private Collection<ResultPoint> possibleResultPoints = new HashSet<ResultPoint>(5);
	private Collection<ResultPoint> lastPossibleResultPoints;

	public ViewfinderView(BarcodeCaptureDialog dialog) {
		super(dialog.getContext());
		mDialog = dialog;
	}

	@Override
	public void onDraw(Canvas canvas) {
		Rect frame = mDialog.mFramingRect;
		if (frame == null) { return; }
		int width = canvas.getWidth();
		int height = canvas.getHeight();

		// Draw the exterior (i.e. outside the framing rect) darkened
		paint.setARGB(OPAQUE / 2, 0, 0, 0);
		canvas.drawRect(0, 0, width, frame.top, paint);
		canvas.drawRect(0, frame.top, frame.left, frame.bottom + 1, paint);
		canvas.drawRect(frame.right + 1, frame.top, width, frame.bottom + 1, paint);
		canvas.drawRect(0, frame.bottom + 1, width, height, paint);

		if (resultBitmap != null) {
			// Draw the opaque result bitmap over the scanning rectangle
			paint.setAlpha(OPAQUE);
			canvas.drawBitmap(resultBitmap, frame.left, frame.top, paint);
		} else {

			// Draw a two pixel solid black border inside the framing rect
			paint.setColor(Color.BLACK);
			canvas.drawRect(frame.left, frame.top, frame.right + 1, frame.top + 2, paint);
			canvas.drawRect(frame.left, frame.top + 2, frame.left + 2, frame.bottom - 1, paint);
			canvas.drawRect(frame.right - 1, frame.top, frame.right + 1, frame.bottom - 1, paint);
			canvas.drawRect(frame.left, frame.bottom - 1, frame.right + 1, frame.bottom + 1, paint);

			// Draw a red "laser scanner" line through the middle to show decoding is active
			paint.setColor(Color.RED);
			paint.setAlpha(SCANNER_ALPHA[scannerAlpha]);
			scannerAlpha = (scannerAlpha + 1) % SCANNER_ALPHA.length;
			int middle = frame.height() / 2 + frame.top;
			canvas.drawRect(frame.left + 2, middle - 1, frame.right - 1, middle + 2, paint);

			Collection<ResultPoint> currentPossible = possibleResultPoints;
			Collection<ResultPoint> currentLast = lastPossibleResultPoints;
			if (currentPossible.isEmpty()) {
				lastPossibleResultPoints = null;
			} else {
				possibleResultPoints = new HashSet<ResultPoint>(5);
				lastPossibleResultPoints = currentPossible;
				paint.setAlpha(OPAQUE);
				paint.setColor(Color.BLUE);
				for (ResultPoint point : currentPossible) {
					canvas.drawCircle(frame.left + point.getX(), frame.top + point.getY(), 6.0f, paint);
				}
			}
			if (currentLast != null) {
				paint.setAlpha(OPAQUE / 2);
				paint.setColor(Color.BLUE);
				for (ResultPoint point : currentLast) {
					canvas.drawCircle(frame.left + point.getX(), frame.top + point.getY(), 3.0f, paint);
				}
			}
			// Request another update at the animation interval, but only repaint the laser line, not the entire viewfinder mask.
			postInvalidateDelayed(ANIMATION_DELAY, frame.left, frame.top, frame.right, frame.bottom);
		}
	}
	public void drawViewfinder() {
		resultBitmap = null;
		invalidate();
	}
	public void drawResultBitmap(Bitmap barcode) {
		resultBitmap = barcode;
		invalidate();
	}
	public void addPossibleResultPoint(ResultPoint point) {
		possibleResultPoints.add(point);
	}

}
