package rexsee.core.animation;

import android.graphics.Camera;
import android.graphics.Matrix;
import android.view.animation.Animation;
import android.view.animation.Transformation;

public class SkewAnimation extends Animation {

	/**
	 * Creates a new 3D rotation on the X axis. The rotation is defined by its
	 * start angle and its end angle. Both angles are in degrees. The rotation
	 * is performed around a center point on the 2D space, defined by a pair
	 * of X and Y coordinates, called centerX and centerY. When the animation
	 * starts, a translation on the Z axis (depth) is performed. The length
	 * of the translation can be specified, as well as whether the translation
	 * should be reversed in time.
	 *
	 * @param fromDegrees the start angle of the 3D rotation
	 * @param toDegrees the end angle of the 3D rotation
	 * @param centerX the X center of the 3D rotation
	 * @param centerY the Y center of the 3D rotation
	 * @param reverse true if the translation should be reversed, false otherwise
	 */

	private final float mStartSkewX;
	private final float mStartSkewY;
	private final float mEndSkewX;
	private final float mEndSkewY;
	private final float mDepthZ;
	private final boolean mReverse;
	private Camera mCamera;

	private float mCenterX;
	private float mCenterY;

	private int mPivotXType = ABSOLUTE;
	private int mPivotYType = ABSOLUTE;
	private float mPivotXValue = 0.0f;
	private float mPivotYValue = 0.0f;

	public SkewAnimation(float startSkewX, float endSkewX, float startSkewY, float endSkewY, float depthZ, boolean reverse, int pivotXType, float pivotXValue,
			int pivotYType, float pivotYValue) {
		mStartSkewX = startSkewX;
		mStartSkewY = startSkewY;
		mEndSkewX = endSkewX;
		mEndSkewY = endSkewY;
		mDepthZ = depthZ;
		mReverse = reverse;
		mPivotXValue = pivotXValue;
		mPivotXType = pivotXType;
		mPivotYValue = pivotYValue;
		mPivotYType = pivotYType;
	}

	@Override
	public void initialize(int width, int height, int parentWidth, int parentHeight) {
		super.initialize(width, height, parentWidth, parentHeight);
		mCenterX = resolveSize(mPivotXType, mPivotXValue, width, parentWidth);
		mCenterY = resolveSize(mPivotYType, mPivotYValue, height, parentHeight);
		mCamera = new Camera();
	}

	@Override
	protected void applyTransformation(float interpolatedTime, Transformation t) {

		float offsetX = mStartSkewX + ((mEndSkewX - mStartSkewX) * interpolatedTime);
		float offsetY = mStartSkewY + ((mEndSkewY - mStartSkewY) * interpolatedTime);

		final float centerX = mCenterX;
		final float centerY = mCenterY;
		final Camera camera = mCamera;

		final Matrix matrix = t.getMatrix();

		camera.save();
		if (mReverse) {
			camera.translate(0.0f, 0.0f, mDepthZ * interpolatedTime);
		} else {
			camera.translate(0.0f, 0.0f, mDepthZ * (1.0f - interpolatedTime));
		}
		camera.getMatrix(matrix);
		camera.restore();

		matrix.preSkew(offsetX, offsetY, -centerX, -centerY);
		matrix.postSkew(offsetX, offsetY, centerX, centerY);

		matrix.preTranslate(-centerX, -centerY);
		matrix.postTranslate(centerX, centerY);

	}

}
