package rexsee.ebook;

import java.util.ArrayList;

import rexsee.core.browser.Browser;
import rexsee.core.style.StyleSheet;
import rexsee.ebook.DrawableLine.DrawableGetter;
import android.app.Activity;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.ColorMatrix;
import android.graphics.ColorMatrixColorFilter;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.graphics.Path;
import android.graphics.Point;
import android.graphics.PointF;
import android.graphics.Region;
import android.graphics.drawable.Drawable;
import android.graphics.drawable.GradientDrawable;
import android.view.MotionEvent;
import android.view.View;
import android.widget.Scroller;
import android.widget.Toast;

public class PageView extends View {

	public static final String EVENT_ONFLIPED = "onFliped";
	public static final String EVENT_ONFIRSTPAGE = "onFlipFirstPage";
	public static final String EVENT_ONLASTPAGE = "onFlipLastPage";

	private static final float[] mColorMatrixArray = {0.55f, 0, 0, 0, 80.0f, 0, 0.55f, 0, 0, 80.0f, 0, 0, 0.55f, 0, 80.0f, 0, 0, 0, 0.2f, 0};
	private static final float[] mMatrixArray = {0, 0, 0, 0, 0, 0, 0, 0, 1.0f};
	private static Drawable mPageBack;
	private static Drawable mBackground;

	private final Scroller mScroller;

	private final Bezier[] bezier;
	private final Path mPath;
	private final PointF mTouch = new PointF();
	private final Point mCorner = new Point();

	protected boolean loadingNull = true;
	protected int loadingIndex = -1;
	protected Bitmap mLoadingPageBitmap = null;
	protected Bitmap mCurPageBitmap = null;
	protected Bitmap mNextPageBitmap = null;
	protected int mPageIndex = 0;

	private final Browser mBrowser;
	public final StyleSheet mStyle;
	private final String mId;
	private String mData;

	protected final String mCachePath;
	protected final DrawableGetter mGetter;
	protected PageFactory mPageFactory;

	public PageView(Browser browser, String id, String style, String cachePath, DrawableGetter getter, Drawable pageBack, Drawable background) {
		this(browser, browser.getContext(), id, style, cachePath, getter, pageBack, background);
	}
	public PageView(Context context, String id, String style, String cachePath, DrawableGetter getter, Drawable pageBack, Drawable background) {
		this(null, context, id, style, cachePath, getter, pageBack, background);
	}
	public PageView(Browser theBrowser, Context context, String id, String style, String cachePath, DrawableGetter getter, Drawable pageBack, Drawable background) {

		super(context);

		mCachePath = cachePath;
		mGetter = getter;
		mBrowser = theBrowser;
		mId = id;
		mStyle = new StyleSheet(context).parseStyle("line-height:150%;text-align:left; text-vertical-align:top;" + style);
		mPageBack = pageBack;
		mBackground = background;

		if (mBrowser != null) {
			mBrowser.eventList.add(EVENT_ONFIRSTPAGE);
			mBrowser.eventList.add(EVENT_ONLASTPAGE);
			mBrowser.eventList.add(EVENT_ONFLIPED);
		}

		bezier = new Bezier[2];
		bezier[0] = new Bezier();
		bezier[1] = new Bezier();
		mCorner.x = 0;
		mCorner.y = 0;
		mPath = new Path();
		mScroller = new Scroller(getContext());
		mTouch.x = 0.01f;
		mTouch.y = 0.01f;

		setClickable(true);
		setOnTouchListener(mOnTouchListener);

	}

	public void error(final String src, final String message) {
		((Activity) getContext()).runOnUiThread(new Runnable() {
			@Override
			public void run() {
				Toast.makeText(getContext(), "PageView." + src + ":" + message, Toast.LENGTH_LONG).show();
			}
		});
	}

	public void load(String data, int startIndex) {

		try {

			mData = data;
			mPageIndex = startIndex;
			loadingNull = true;

			mPageFactory = new PageFactory(this, mStyle, mCachePath, mGetter, mBackground);
			mLoadingPageBitmap = mPageFactory.getLoadingBitmap("Loading......");
			mCurPageBitmap = mLoadingPageBitmap;
			mNextPageBitmap = mLoadingPageBitmap;
			this.postInvalidate();

		} catch (Exception e) {
			error("load", e.getLocalizedMessage());
		}

		try {
			mPageFactory.parse(data);
		} catch (Exception e) {
			error("load.parse", e.getLocalizedMessage());
		}

	}
	public String getData() {
		return mData;
	}

	public void clear() {
		if (mPageFactory != null) {
			mPageFactory.clear();
			mPageFactory = null;
		}
		mData = null;
		mPageIndex = 0;
		loadingIndex = -1;
		this.postInvalidate();
	}
	public void reload() {
		try {
			if (mPageFactory == null) return;
			loadingIndex++;
			if (loadingNull && loadingIndex >= mPageIndex) {
				mCurPageBitmap = mPageFactory.getPage(mPageIndex);
				this.postInvalidate();
				loadingNull = false;
			}
		} catch (Exception e) {
			error("reload", e.getLocalizedMessage());
		}
	}
	public void setPageIndex(int index) {
		if (mPageFactory == null) return;
		if (index < 0) return;
		if (!mPageFactory.isLoading() && index >= mPageFactory.getTotal()) return;
		if (index == mPageIndex) return;
		try {
			abortAnimation();
			mCurPageBitmap = mPageFactory.getPage(mPageIndex);
			mNextPageBitmap = mPageFactory.getPage(index);
			if (mNextPageBitmap == null) {
				mNextPageBitmap = mLoadingPageBitmap;
				loadingNull = true;
			}
			if (index > mPageIndex) {
				startAnimation(1, 1200);
			} else {
				startAnimation(2, 1200);
			}
			mPageIndex = index;
			this.postInvalidate();
		} catch (Exception e) {
			error("setPageIndex", e.getLocalizedMessage());
		}
	}
	public int getPageIndex() {
		return mPageIndex;
	}
	public int getPageTotal() {
		if (mPageFactory == null) return -1;
		return mPageFactory.getTotal();
	}

	public String getPageTitle() {
		return getPageTitle(mPageIndex);
	}
	public String getPageTitle(int index) {
		ArrayList<Page> pages = mPageFactory.getPages();
		if (index < 0 || index >= pages.size()) return "";
		Page page = pages.get(index);
		String rtn = "";
		rtn += "{";
		rtn += "\"page\":" + (page.index + 1);
		rtn += ",\"title\":\"" + getTitles(page) + "\"";
		rtn += ",\"inheriteTitle\":\"" + ((page.inheriteTitle == null) ? "" : page.inheriteTitle) + "\"";
		rtn += ",\"total\":" + pages.size();
		rtn += "}";
		return rtn;
	}

	public ArrayList<Page> search(String text) {
		if (mPageFactory == null) return null;
		return mPageFactory.searchText(text);
	}
	public ArrayList<Page> getIndexes() {
		return mPageFactory.getPages();
	}
	public String getPageIndexes() {
		ArrayList<Page> pages = mPageFactory.getPages();
		String rtn = "";
		for (int i = 0; i < pages.size(); i++) {
			Page page = pages.get(i);
			if (page.indexTitles.size() == 0) continue;
			for (int j = 0; j < page.indexTitles.size(); j++) {
				if (!rtn.equals("")) rtn += ",";
				rtn += "{\"page\":" + (i + 1) + ",\"title\":\"" + page.indexTitles.get(i) + "\"}";
			}
		}
		return "[" + rtn + "]";
	}
	public String searchText(String text) {
		ArrayList<Page> pages = search(text);
		if (pages == null || pages.size() == 0) return "[]";
		String rtn = "";
		for (int i = 0; i < pages.size(); i++) {
			Page page = pages.get(i);
			if (!rtn.equals("")) rtn += ",";
			rtn += "{";
			rtn += "\"page\":" + (page.index + 1);
			rtn += ",\"title\":\"" + getTitles(page) + "\"";
			rtn += ",\"inheriteTitle\":\"" + ((page.inheriteTitle == null) ? "" : page.inheriteTitle) + "\"";
			rtn += "}";
		}
		return "[" + rtn + "]";
	}

	private String getTitles(Page page) {
		String titles = "";
		for (int i = 0; i < page.indexTitles.size(); i++) {
			if (i > 0) titles += ",";
			titles += page.indexTitles.get(i);
		}
		return titles;
	}

	@Override
	protected void onDraw(Canvas canvas) {
		try {
			if (mPageFactory == null) return;
			if (mTouch.x == -100) {
				mCurPageBitmap = mPageFactory.getPage(mPageIndex);
				canvas.drawBitmap(mCurPageBitmap, 0, 0, null);
				loadingNull = false;
				return;
			}

			mPageBack.setBounds(0, 0, this.getWidth(), this.getHeight());
			mPageBack.draw(canvas);

			calcPoints();
			drawCurrentPageArea(canvas);
			drawNextPageAreaAndShadow(canvas);
			drawCurrentPageShadow(canvas);
			drawCurrentBackArea(canvas);
			View root = getRootView();
			if (root != null && !root.equals(PageView.this)) root.postInvalidate();

		} catch (Exception e) {
			error("onDraw", e.getLocalizedMessage());
		}
	}
	@Override
	public void computeScroll() {
		if (mPageFactory == null) return;
		super.computeScroll();
		try {
			if (mScroller.computeScrollOffset()) {
				float x = mScroller.getCurrX();
				float y = mScroller.getCurrY();
				if (mScroller.getFinalX() == x && mScroller.getFinalY() == y) {
					if (mBrowser != null) mBrowser.eventList.run(EVENT_ONFLIPED, new String[]{mId, String.valueOf(getPageIndex()), String.valueOf(mPageFactory.getTotal())});
					mTouch.x = -100;
				} else {
					mTouch.x = x;
					mTouch.y = y;
				}
				postInvalidate();
			} else {
			}
		} catch (Exception e) {
			error("computeScroll", e.getLocalizedMessage());
		}
	}

	public void startAnimation(int duration) {
		try {
			// dx:ˮƽ򻬶ľ룬ֵʹ; dy:ֱ򻬶ľ룬ֵʹϹ
			int mWidth = this.getWidth();
			int mHeight = this.getHeight();
			int dx = (mCorner.x > 0) ? -(int) (mWidth + mTouch.x - 0.1f) : (int) (mWidth - mTouch.x + mWidth - 0.1f);
			int dy = (mCorner.y > 0) ? (int) (mHeight - mTouch.y - 0.1f) : (int) (1 - mTouch.y - 0.1f); // ֹmTouch.yձΪ0
			mScroller.startScroll((int) mTouch.x, (int) mTouch.y, dx, dy, duration);
		} catch (Exception e) {
			error("startAnimation", e.getLocalizedMessage());
		}
	}
	public void startAnimation(int direction, int duration) {
		abortAnimation();
		int w = this.getWidth();
		int h = this.getHeight();
		switch (direction) {
			case 1 :
				mTouch.x = w * 2f / 3f;
				mTouch.y = h * 2f / 3f;
				break;
			case 2 :
				mTouch.x = w / 3f;
				mTouch.y = h * 2f / 3f;
				break;
			case 3 :
				mTouch.x = w / 3f;
				mTouch.y = h / 3f;
				break;
			default :
				mTouch.x = w * 2f / 3f;
				mTouch.y = h / 3f;
				break;
		}
		mCorner.x = (mTouch.x <= w / 2) ? 0 : w;
		mCorner.y = (mTouch.y <= h / 2) ? 0 : h;
		startAnimation(duration);
	}
	public void abortAnimation() {
		if (!mScroller.isFinished()) {
			mScroller.abortAnimation();
		}
	}

	//Default OnTouchLisener
	private final OnTouchListener mOnTouchListener = new OnTouchListener() {
		@Override
		public boolean onTouch(View v, MotionEvent event) {
			if (mPageFactory == null) return true;
			if (loadingIndex == 0) return true;
			int action = event.getAction();
			float x = event.getX();
			float y = event.getY();
			int w = v.getWidth();
			int h = v.getHeight();
			if (action == MotionEvent.ACTION_DOWN) {
				try {
					abortAnimation();
					mCorner.x = (x <= w / 2) ? 0 : w;
					mCorner.y = (y <= h / 2) ? 0 : h;
					if (mCorner.x == 0) {
						if (mPageIndex <= 0) {
							if (mBrowser != null) mBrowser.eventList.run(EVENT_ONFIRSTPAGE, new String[]{mId});
							mTouch.x = -100;
							return false;
						} else {
							mCurPageBitmap = mPageFactory.getPage(mPageIndex);
							mNextPageBitmap = mPageFactory.getPage(mPageIndex - 1);
						}
					} else {
						if (mPageIndex >= loadingIndex) {
							mTouch.x = -100;
							if (mBrowser != null) mBrowser.eventList.run(EVENT_ONLASTPAGE, new String[]{mId});
							return false;
						} else {
							mCurPageBitmap = mPageFactory.getPage(mPageIndex);
							mNextPageBitmap = mPageFactory.getPage(mPageIndex + 1);
						}
					}
					mTouch.x = x;
					mTouch.y = y;
				} catch (Exception e) {
					error("onTouchDown", e.getLocalizedMessage());
				}
			} else if (action == MotionEvent.ACTION_MOVE) {
				try {
					if ((mCorner.x == 0 && mPageIndex <= 0) || (mCorner.x > 0 && mPageIndex >= loadingIndex)) return true;
					mTouch.x = x;
					if (y > 0 && y < h) mTouch.y = y;
				} catch (Exception e) {
					error("onTouchMove", e.getLocalizedMessage());
				}
			} else if (action == MotionEvent.ACTION_UP) {
				try {
					if ((mCorner.x == 0 && mPageIndex <= 0) || (mCorner.x > 0 && mPageIndex >= loadingIndex)) return true;
					if (Math.abs(mTouch.x - mCorner.x) > w / 2) {
						mPageIndex = (mCorner.x == 0) ? mPageIndex - 1 : mPageIndex + 1;
						startAnimation(1200);
					} else {
						mTouch.x = mCorner.x - 0.09f;
						mTouch.y = mCorner.y - 0.09f;
					}
				} catch (Exception e) {
					error("onTouchUp", e.getLocalizedMessage());
				}
			}
			v.postInvalidate();
			return true;
		}
	};

	//Calculations
	private void calcPoints() {
		int mWidth = this.getWidth();
		PointF mMiddle = new PointF();
		mMiddle.x = (mTouch.x + mCorner.x) / 2;
		mMiddle.y = (mTouch.y + mCorner.y) / 2;
		bezier[0].control.x = mMiddle.x - (mCorner.y - mMiddle.y) * (mCorner.y - mMiddle.y) / (mCorner.x - mMiddle.x);
		bezier[0].control.y = mCorner.y;
		bezier[1].control.x = mCorner.x;
		bezier[1].control.y = mMiddle.y - (mCorner.x - mMiddle.x) * (mCorner.x - mMiddle.x) / (mCorner.y - mMiddle.y);
		bezier[0].start.x = bezier[0].control.x - (mCorner.x - bezier[0].control.x) / 2;
		bezier[0].start.y = mCorner.y;
		// bezier[0].start.x < 0bezier[0].start.x > 480ʱ, ҳBUGڴ
		if (mTouch.x > 0 && mTouch.x < mWidth) {
			if (bezier[0].start.x < 0 || bezier[0].start.x > mWidth) {
				if (bezier[0].start.x < 0) {
					bezier[0].start.x = mWidth - bezier[0].start.x;
				}
				float f1 = Math.abs(mCorner.x - mTouch.x);
				float f2 = mWidth * f1 / bezier[0].start.x;
				mTouch.x = Math.abs(mCorner.x - f2);
				float f3 = Math.abs(mCorner.x - mTouch.x) * Math.abs(mCorner.y - mTouch.y) / f1;
				mTouch.y = Math.abs(mCorner.y - f3);
				mMiddle.x = (mTouch.x + mCorner.x) / 2;
				mMiddle.y = (mTouch.y + mCorner.y) / 2;
				bezier[0].control.x = mMiddle.x - (mCorner.y - mMiddle.y) * (mCorner.y - mMiddle.y) / (mCorner.x - mMiddle.x);
				bezier[0].control.y = mCorner.y;
				bezier[1].control.x = mCorner.x;
				bezier[1].control.y = mMiddle.y - (mCorner.x - mMiddle.x) * (mCorner.x - mMiddle.x) / (mCorner.y - mMiddle.y);
				bezier[0].start.x = bezier[0].control.x - (mCorner.x - bezier[0].control.x) / 2;
			}
		}
		bezier[1].start.x = mCorner.x;
		bezier[1].start.y = bezier[1].control.y - (mCorner.y - bezier[1].control.y) / 2;
		bezier[0].end = getCross(mTouch, bezier[0].control, bezier[0].start, bezier[1].start);
		bezier[1].end = getCross(mTouch, bezier[1].control, bezier[0].start, bezier[1].start);
		bezier[0].vertex.x = (bezier[0].start.x + 2 * bezier[0].control.x + bezier[0].end.x) / 4;
		bezier[0].vertex.y = (2 * bezier[0].control.y + bezier[0].start.y + bezier[0].end.y) / 4;
		bezier[1].vertex.x = (bezier[1].start.x + 2 * bezier[1].control.x + bezier[1].end.x) / 4;
		bezier[1].vertex.y = (2 * bezier[1].control.y + bezier[1].start.y + bezier[1].end.y) / 4;
	}

	private void drawCurrentPageArea(Canvas canvas) {
		if (mCurPageBitmap == null) { return; }
		try {
			mPath.reset();
			mPath.moveTo(bezier[0].start.x, bezier[0].start.y);
			mPath.quadTo(bezier[0].control.x, bezier[0].control.y, bezier[0].end.x, bezier[0].end.y);
			mPath.lineTo(mTouch.x, mTouch.y);
			mPath.lineTo(bezier[1].end.x, bezier[1].end.y);
			mPath.quadTo(bezier[1].control.x, bezier[1].control.y, bezier[1].start.x, bezier[1].start.y);
			mPath.lineTo(mCorner.x, mCorner.y);
			mPath.close();
			canvas.save();
			try {
				canvas.clipPath(mPath, Region.Op.XOR);
			} catch (Exception e) {
			}
			canvas.drawBitmap(mCurPageBitmap, 0, 0, null);
			canvas.restore();
		} catch (Exception e) {
			error("drawCurrentPageArea", e.getLocalizedMessage());
		}
	}
	private void drawCurrentBackArea(Canvas canvas) {
		if (mCurPageBitmap == null) { return; }
		try {
			int i = (int) (bezier[0].start.x + bezier[0].control.x) / 2;
			float f1 = Math.abs(i - bezier[0].control.x);
			int i1 = (int) (bezier[1].start.y + bezier[1].control.y) / 2;
			float f2 = Math.abs(i1 - bezier[1].control.y);
			float f3 = Math.min(f1, f2);
			Path path = new Path();
			path.reset();
			path.moveTo(bezier[1].vertex.x, bezier[1].vertex.y);
			path.lineTo(bezier[0].vertex.x, bezier[0].vertex.y);
			path.lineTo(bezier[0].end.x, bezier[0].end.y);
			path.lineTo(mTouch.x, mTouch.y);
			path.lineTo(bezier[1].end.x, bezier[1].end.y);
			path.close();
			GradientDrawable mFolderShadowDrawable;
			int left;
			int right;
			boolean mIsRTandLB = ((mCorner.x == 0 && mCorner.y == this.getHeight()) || (mCorner.x == this.getWidth() && mCorner.y == 0)) ? true : false;
			PageDrawables pageDrawables = new PageDrawables();
			if (mIsRTandLB) {
				left = (int) (bezier[0].start.x - 1);
				right = (int) (bezier[0].start.x + f3 + 1);
				mFolderShadowDrawable = pageDrawables.folder_LR;
			} else {
				left = (int) (bezier[0].start.x - f3 - 1);
				right = (int) (bezier[0].start.x + 1);
				mFolderShadowDrawable = pageDrawables.folder_RL;
			}
			canvas.save();
			try {
				canvas.clipPath(mPath);
			} catch (Exception e) {
			}
			try {
				canvas.clipPath(path, Region.Op.INTERSECT);
			} catch (Exception e) {
			}

			ColorMatrix cm = new ColorMatrix();
			cm.set(mColorMatrixArray);
			Paint mPaint = new Paint();
			mPaint.setStyle(Paint.Style.FILL);
			mPaint.setColorFilter(new ColorMatrixColorFilter(cm));

			float dis = (float) Math.hypot(mCorner.x - bezier[0].control.x, bezier[1].control.y - mCorner.y);
			float f8 = (mCorner.x - bezier[0].control.x) / dis;
			float f9 = (bezier[1].control.y - mCorner.y) / dis;
			float[] array = mMatrixArray;
			array[0] = 1 - 2 * f9 * f9;
			array[1] = 2 * f8 * f9;
			array[3] = array[1];
			array[4] = 1 - 2 * f8 * f8;
			Matrix mMatrix = new Matrix();
			mMatrix.reset();
			mMatrix.setValues(array);
			mMatrix.preTranslate(-bezier[0].control.x, -bezier[0].control.y);
			mMatrix.postTranslate(bezier[0].control.x, bezier[0].control.y);
			canvas.drawBitmap(mCurPageBitmap, mMatrix, mPaint);
			mPaint.setColorFilter(null);
			float mDegrees = (float) Math.toDegrees(Math.atan2(bezier[0].control.x - mCorner.x, bezier[1].control.y - mCorner.y));
			canvas.rotate(mDegrees, bezier[0].start.x, bezier[0].start.y);
			float mMaxLength = (float) Math.hypot(this.getWidth(), this.getHeight());
			mFolderShadowDrawable.setBounds(left, (int) bezier[0].start.y, right, (int) (bezier[0].start.y + mMaxLength));
			mFolderShadowDrawable.draw(canvas);
			canvas.restore();
		} catch (Exception e) {
			error("drawCurrentBackArea", e.getLocalizedMessage());
		}
	}
	private void drawCurrentPageShadow(Canvas canvas) {
		try {
			boolean mIsRTandLB = ((mCorner.x == 0 && mCorner.y == this.getHeight()) || (mCorner.x == this.getWidth() && mCorner.y == 0)) ? true : false;
			double degree = (mIsRTandLB) ? Math.PI / 4 - Math.atan2(bezier[0].control.y - mTouch.y, mTouch.x - bezier[0].control.x) : Math.PI / 4 - Math.atan2(mTouch.y - bezier[0].control.y, mTouch.x - bezier[0].control.x);
			double d1 = 25 * 1.414 * Math.cos(degree);
			double d2 = 25 * 1.414 * Math.sin(degree);
			float x = (float) (mTouch.x + d1);
			float y = (mIsRTandLB) ? (float) (mTouch.y + d2) : (float) (mTouch.y - d2);
			Path path = new Path();
			path.reset();
			path.moveTo(x, y);
			path.lineTo(mTouch.x, mTouch.y);
			path.lineTo(bezier[0].control.x, bezier[0].control.y);
			path.lineTo(bezier[0].start.x, bezier[0].start.y);
			path.close();
			float rotateDegrees;
			canvas.save();

			try {
				canvas.clipPath(mPath, Region.Op.XOR);
			} catch (Exception e) {
			}
			try {
				canvas.clipPath(path, Region.Op.INTERSECT);
			} catch (Exception e) {
			}
			int leftx;
			int rightx;
			PageDrawables pageDrawables = new PageDrawables();
			GradientDrawable mCurrentPageShadow;
			if (mIsRTandLB) {
				leftx = (int) (bezier[0].control.x);
				rightx = (int) bezier[0].control.x + 25;
				mCurrentPageShadow = pageDrawables.front_VLR;
			} else {
				leftx = (int) (bezier[0].control.x - 25);
				rightx = (int) bezier[0].control.x + 1;
				mCurrentPageShadow = pageDrawables.front_VRL;
			}

			rotateDegrees = (float) Math.toDegrees(Math.atan2(mTouch.x - bezier[0].control.x, bezier[0].control.y - mTouch.y));
			canvas.rotate(rotateDegrees, bezier[0].control.x, bezier[0].control.y);
			float mMaxLength = (float) Math.hypot(this.getWidth(), this.getHeight());
			mCurrentPageShadow.setBounds(leftx, (int) (bezier[0].control.y - mMaxLength), rightx, (int) (bezier[0].control.y));
			mCurrentPageShadow.draw(canvas);
			canvas.restore();

			path.reset();
			path.moveTo(x, y);
			path.lineTo(mTouch.x, mTouch.y);
			path.lineTo(bezier[1].control.x, bezier[1].control.y);
			path.lineTo(bezier[1].start.x, bezier[1].start.y);
			path.close();
			canvas.save();
			try {
				canvas.clipPath(mPath, Region.Op.XOR);
			} catch (Exception e) {
			}
			try {
				canvas.clipPath(path, Region.Op.INTERSECT);
			} catch (Exception e) {
			}
			if (mIsRTandLB) {
				leftx = (int) (bezier[1].control.y);
				rightx = (int) (bezier[1].control.y + 25);
				mCurrentPageShadow = pageDrawables.front_HTB;
			} else {
				leftx = (int) (bezier[1].control.y - 25);
				rightx = (int) (bezier[1].control.y + 1);
				mCurrentPageShadow = pageDrawables.front_HBT;
			}
			rotateDegrees = (float) Math.toDegrees(Math.atan2(bezier[1].control.y - mTouch.y, bezier[1].control.x - mTouch.x));
			canvas.rotate(rotateDegrees, bezier[1].control.x, bezier[1].control.y);
			float temp = (bezier[1].control.y < 0) ? bezier[1].control.y - this.getHeight() : bezier[1].control.y;
			int hmg = (int) Math.hypot(bezier[1].control.x, temp);
			if (hmg > mMaxLength) {
				mCurrentPageShadow.setBounds((int) (bezier[1].control.x - 25) - hmg, leftx, (int) (bezier[1].control.x + mMaxLength) - hmg, rightx);
			} else {
				mCurrentPageShadow.setBounds((int) (bezier[1].control.x - mMaxLength), leftx, (int) (bezier[1].control.x), rightx);
			}
			mCurrentPageShadow.draw(canvas);
			canvas.restore();
		} catch (Exception e) {
			error("drawCurrentPageShadow", e.getLocalizedMessage());
		}
	}
	private void drawNextPageAreaAndShadow(Canvas canvas) {
		if (mNextPageBitmap == null) { return; }
		try {
			Path path = new Path();
			path.reset();
			path.moveTo(bezier[0].start.x, bezier[0].start.y);
			path.lineTo(bezier[0].vertex.x, bezier[0].vertex.y);
			path.lineTo(bezier[1].vertex.x, bezier[1].vertex.y);
			path.lineTo(bezier[1].start.x, bezier[1].start.y);
			path.lineTo(mCorner.x, mCorner.y);
			path.close();

			float mDegrees = (float) Math.toDegrees(Math.atan2(bezier[0].control.x - mCorner.x, bezier[1].control.y - mCorner.y));
			int leftx;
			int rightx;
			boolean mIsRTandLB = ((mCorner.x == 0 && mCorner.y == this.getHeight()) || (mCorner.x == this.getWidth() && mCorner.y == 0)) ? true : false;
			float mTouchToCornerDis = (float) Math.hypot((mTouch.x - mCorner.x), (mTouch.y - mCorner.y));
			GradientDrawable mBackShadowDrawable;
			PageDrawables pageDrawables = new PageDrawables();
			if (mIsRTandLB) {
				leftx = (int) (bezier[0].start.x);
				rightx = (int) (bezier[0].start.x + mTouchToCornerDis / 4);
				mBackShadowDrawable = pageDrawables.back_LR;
			} else {
				leftx = (int) (bezier[0].start.x - mTouchToCornerDis / 4);
				rightx = (int) bezier[0].start.x;
				mBackShadowDrawable = pageDrawables.back_RL;
			}
			canvas.save();
			try {
				canvas.clipPath(mPath);
			} catch (Exception e) {
			}
			try {
				canvas.clipPath(path, Region.Op.INTERSECT);
			} catch (Exception e) {
			}
			canvas.drawBitmap(mNextPageBitmap, 0, 0, null);
			canvas.rotate(mDegrees, bezier[0].start.x, bezier[0].start.y);
			float mMaxLength = (float) Math.hypot(this.getWidth(), this.getHeight());
			mBackShadowDrawable.setBounds(leftx, (int) bezier[0].start.y, rightx, (int) (mMaxLength + bezier[0].start.y));
			mBackShadowDrawable.draw(canvas);
			canvas.restore();
		} catch (Exception e) {
			error("drawNextPageAreaAndShadow", e.getLocalizedMessage());
		}

	}
	private PointF getCross(PointF P1, PointF P2, PointF P3, PointF P4) {
		//ֱP1P2ֱP3P4Ľ
		PointF CrossP = new PointF();
		// Ԫͨʽ y=ax+b
		float a1 = (P2.y - P1.y) / (P2.x - P1.x);
		float b1 = ((P1.x * P2.y) - (P2.x * P1.y)) / (P1.x - P2.x);
		float a2 = (P4.y - P3.y) / (P4.x - P3.x);
		float b2 = ((P3.x * P4.y) - (P4.x * P3.y)) / (P3.x - P4.x);
		CrossP.x = (b2 - b1) / (a1 - a2);
		CrossP.y = a1 * CrossP.x + b1;
		return CrossP;
	}

	public void destroy() {
		if (mLoadingPageBitmap != null) {
			mLoadingPageBitmap.recycle();
			mLoadingPageBitmap = null;
		}
		if (mCurPageBitmap != null) {
			mCurPageBitmap.recycle();
			mCurPageBitmap = null;
		}
		if (mNextPageBitmap != null) {
			mNextPageBitmap.recycle();
			mNextPageBitmap = null;
		}
		if (mPageFactory != null) {
			mPageFactory.destroy();
			mPageFactory = null;
		}
		System.gc();
	}

}
