package rexsee.sensor;

import rexsee.core.browser.Browser;
import rexsee.core.browser.clazz.JavascriptInterface;
import android.content.Context;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;

public class RexseeSensorAccelerometer implements JavascriptInterface {

	public static final String INTERFACE_NAME = "Accelerometer";
	@Override
	public String getInterfaceName() {
		return mBrowser.application.resources.prefix + INTERFACE_NAME;
	}
	@Override
	public JavascriptInterface getInheritInterface(Browser childBrowser) {
		return this;
	}
	@Override
	public JavascriptInterface getNewInterface(Browser childBrowser) {
		return new RexseeSensorAccelerometer(childBrowser);
	}

	public static final String EVENT_ONACCELERATE = "onAccelerometerChanged";

	private final Context mContext;
	private final Browser mBrowser;
	private final SensorManager mSensorManager;
	private final SensorEventListener mSensorListener;
	private final Sensor mSensor;

	private int mRate = SensorManager.SENSOR_DELAY_NORMAL;
	private int mCycle = 100;
	private int mEventCycle = 100;

	private float mAccuracyX = 0;
	private float mAccuracyY = 0;
	private float mAccuracyZ = 0;

	private long lastUpdate = -1;
	private long lastEvent = -1;

	private float x = -999, y = -999, z = -999;

	public RexseeSensorAccelerometer(Browser browser) {

		mContext = browser.getContext();
		mBrowser = browser;
		browser.eventList.add(EVENT_ONACCELERATE);

		mSensorManager = (SensorManager) mContext.getSystemService(Context.SENSOR_SERVICE);

		mSensor = mSensorManager.getDefaultSensor(Sensor.TYPE_ACCELEROMETER);

		mSensorListener = new SensorEventListener() {
			@Override
			public void onAccuracyChanged(Sensor sensor, int accuracy) {
			}
			@Override
			public void onSensorChanged(SensorEvent event) {
				if (event.sensor.getType() != Sensor.TYPE_ACCELEROMETER) return;
				long curTime = System.currentTimeMillis();
				if (lastUpdate == -1 || (curTime - lastUpdate) > mCycle) {
					lastUpdate = curTime;
					float lastX = x;
					float lastY = y;
					float lastZ = z;
					x = event.values[SensorManager.DATA_X];
					y = event.values[SensorManager.DATA_Y];
					z = event.values[SensorManager.DATA_Z];
					if (lastEvent == -1 || (curTime - lastEvent) > mEventCycle) {
						if (
								(mAccuracyX >= 0 && Math.abs(x - lastX) > mAccuracyX)
										|| (mAccuracyY >= 0 && Math.abs(y - lastY) > mAccuracyY)
										|| (mAccuracyZ >= 0 && Math.abs(z - lastZ) > mAccuracyZ)
							) {
								lastEvent = curTime;
								mBrowser.eventList.run(EVENT_ONACCELERATE);
							}
						}
					}
				}
		};

	}

	public String getLastKnownX() {
		return (x == -999) ? "null" : String.valueOf(x);
	}
	public String getLastKnownY() {
		return (y == -999) ? "null" : String.valueOf(y);
	}
	public String getLastKnownZ() {
		return (z == -999) ? "null" : String.valueOf(z);
	}

	public void setRate(String rate) {
		mRate = SensorRate.getInt(rate);
	}
	public String getRate() {
		return SensorRate.getString(mRate);
	}
	public void setCycle(int milliseconds) {
		mCycle = milliseconds;
	}
	public int getCycle() {
		return mCycle;
	}
	public void setEventCycle(int milliseconds) {
		mEventCycle = milliseconds;
	}
	public int getEventCycle() {
		return mEventCycle;
	}
	public void setAccuracy(float x, float y, float z) {
		mAccuracyX = x;
		mAccuracyY = y;
		mAccuracyZ = z;
	}
	public float getAccuracyX() {
		return mAccuracyX;
	}
	public float getAccuracyY() {
		return mAccuracyY;
	}
	public float getAccuracyZ() {
		return mAccuracyZ;
	}

	public boolean isReady() {
		return (mSensor == null) ? false : true;
	}
	public void start() {
		if (isReady()) {
			mSensorManager.registerListener(mSensorListener, mSensor, mRate);
		} else {
			mBrowser.exception(getInterfaceName(), "Accelerometer is not found.");
		}
	}
	public void stop() {
		if (isReady()) {
			mSensorManager.unregisterListener(mSensorListener);
		}
	}

}
